@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -87,8 +87,7 @@ static int setaudio(struct xmp_options *o)
 	int bsize = 32 * 1024;
 	ALconfig config;
 	long pvbuffer[2];
-	char *token;
-	char **parm = o->parm;
+	char *token, **parm;
 	int i;
 
 	parm_init();
@@ -206,12 +205,12 @@ static int setaudio(struct xmp_options *o)
 	if ((audio_port = ALopenport("xmp", "w", config)) == 0)
 		return XMP_ERR_DINIT;
 
-	return XMP_OK;
+	return 0;
 }
 
 static int init(struct xmp_context *ctx)
 {
-	if (setaudio(&ctx->o) != XMP_OK)
+	if (setaudio(&ctx->o) != 0)
 		return XMP_ERR_DINIT;
 
 	return xmp_smix_on(ctx);
