@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -100,8 +100,7 @@ static int setaudio(struct xmp_options *o)
 	int gain;
 	int bsize = 32 * 1024;
 	int port;
-	char *token;
-	char **parm = ctl->o;
+	char *token, **parm;
 	AUDIO_INITINFO(&ainfo);
 
 	/* try to open audioctl device */
@@ -185,7 +184,7 @@ static int setaudio(struct xmp_options *o)
 		drv_solaris.description = "Solaris CS4231 PCM audio";
 	}
 
-	return XMP_OK;
+	return 0;
 }
 
 static int init(struct xmp_context *ctx)
@@ -195,7 +194,7 @@ static int init(struct xmp_context *ctx)
 	if ((audio_fd = open("/dev/audio", O_WRONLY)) == -1)
 		return XMP_ERR_DINIT;
 
-	if (setaudio(o) != XMP_OK)
+	if (setaudio(o) != 0)
 		return XMP_ERR_DINIT;
 
 	return xmp_smix_on(ctx);
@@ -213,7 +212,7 @@ static void bufdump(struct xmp_context *ctx, int i)
 	while (i) {
 		if ((j = write(audio_fd, b, i)) > 0) {
 			i -= j;
-			(char *)b += j;
+			b = (char *)b + j;
 		} else
 			break;
 	}
