@@ -7,7 +7,7 @@
 
 #define SMIX_NUMVOC	64	/* default number of softmixer voices */
 #define SMIX_MAXRATE	48000	/* max sampling rate (Hz) */
-#define SMIX_MINBPM	0x10	/* min BPM */
+#define SMIX_MINBPM	5	/* min BPM */
 #define SMIX_RESMAX	(sizeof (int16))	/* max output resolution */
 
 #define SMIX_SHIFT	16
@@ -21,6 +21,7 @@
 #define SLOW_ATTACK	64
 #define SLOW_RELEASE	16
 
+
 struct voice_info {
 	int chn;		/* channel link */
 	int root;		/* */
@@ -56,16 +57,17 @@ struct voice_info {
 };
 
 int	xmp_smix_on		(struct xmp_context *);
-void	xmp_smix_off		(void);
+void	xmp_smix_off		(struct xmp_context *);
 void    xmp_smix_setvol		(struct xmp_context *, int, int);
 void    xmp_smix_seteffect	(struct xmp_context *, int, int, int);
 void    xmp_smix_setpan		(struct xmp_context *, int, int);
-int	xmp_smix_numvoices	(int);
-void	xmp_smix_echoback	(int);
+int	xmp_smix_numvoices	(struct xmp_context *, int);
+void	xmp_smix_echoback	(struct xmp_context *, int);
 void	xmp_smix_starttimer	(void);
 void	xmp_smix_stoptimer	(void);
+int	xmp_smix_softmixer	(struct xmp_context *);
 int	xmp_smix_writepatch	(struct xmp_context *, struct patch_info *);
-int	xmp_smix_getmsg		(void);
+int	xmp_smix_getmsg		(struct xmp_context *);
 void    *xmp_smix_buffer	(struct xmp_context *);
 
 #endif /* __MIXER_H */
