@@ -16,10 +16,10 @@
 #define AMIGA_LIMIT_UPPER 108
 #define AMIGA_LIMIT_LOWER 907
 
-int	note_to_period		(int, int, int);
+double	note_to_period		(int, int, int);
 int	note_to_period_mix	(int, int);
 int	period_to_note		(int);
-int	period_to_bend		(int, int, int, int, int);
+int	period_to_bend		(double, int, int, int, int);
 void	c2spd_to_note		(int, int *, int *);
 
 #endif /* __PERIOD_H */
