@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -42,7 +42,8 @@ static int digi_test(FILE *f, char *t, const int start)
 {
     char buf[20];
 
-    fread(buf, 20, 1, f);
+    if (fread(buf, 1, 20, f) < 20)
+	return -1;
 
     if (memcmp(buf, "DIGI Booster module", 19))
 	return -1;
