@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -10,6 +10,8 @@
 #include "config.h"
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
 #include "load.h"
 
 #define MAGIC_Funk	MAGIC4('F','u','n','k')
@@ -28,6 +30,8 @@ struct xmp_loader_info fnk_loader = {
 static int fnk_test(FILE *f, char *t, const int start)
 {
     uint8 a, b;
+    int size;
+    struct stat st;
 
     if (read32b(f) != MAGIC_Funk)
 	return -1;
@@ -43,6 +47,14 @@ static int fnk_test(FILE *f, char *t, const int start)
     if (MSN(b) > 7 || LSN(b) > 9)	/* CPU and card */
 	return -1;
 
+    size = read32l(f);
+    if (size < 1024)
+	return -1;
+
+    fstat(fileno(f), &st);
+    if (size != st.st_size)
+	return -1;
+
     read_title(f, t, 0);
 
     return 0;
