@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr.
+ * Copyright (C) 1996-2011 Claudio Matsuoka and Hipolito Carraro Jr.
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -29,6 +29,15 @@ struct xmp_loader_info it_loader = {
     it_load
 };
 
+#ifdef WIN32
+// FIXME: not thread-safe
+struct tm *localtime_r(const time_t *timep, struct tm *result)
+{
+    memcpy(result, localtime(timep), sizeof(struct tm));
+    return result;
+}
+#endif
+
 static int it_test(FILE *f, char *t, const int start)
 {
     if (read32b(f) != MAGIC_IMPM)
@@ -261,7 +270,6 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
     int max_ch, flag;
     int inst_map[120], inst_rmap[XXM_KEY_MAX];
     char tracker_name[80];
-    int mpt = 0;	/* ModPlug Tracker has quirks */
 
     LOAD_INIT();
 
@@ -374,7 +382,6 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	    sprintf(tracker_name, "ModPlug Tracker 1.16");
 	    /* ModPlug Tracker files aren't really IMPM 2.00 */
 	    ifh.cmwt = ifh.flags & IT_USE_INST ? 0x214 : 0x100;	
-	    mpt = 1;
 	} else if (ifh.cwt == 0x0216) {
 	    sprintf(tracker_name, "Impulse Tracker 2.14v3");
 	} else if (ifh.cwt == 0x0217) {
@@ -387,8 +394,11 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	}
 	break;
     case 0x08:
+    case 0x7f:
 	if (ifh.cwt == 0x0888) {
 	    sprintf(tracker_name, "OpenMPT 1.17+");
+	} else if (ifh.cwt == 0x7fff) {
+	    sprintf(tracker_name, "munch.py");
 	} else {
 	    sprintf(tracker_name, "unknown (%04x)", ifh.cwt);
 	}
@@ -734,7 +744,7 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	ish.magic = read32b(f);
 	fread(&ish.dosname, 12, 1, f);
 	ish.zero = read8(f);
-	ish.gvl = mpt ? read8(f),0x40 : read8(f);
+	ish.gvl = read8(f);
 	ish.flags = read8(f);
 	ish.vol = read8(f);
 	fread(&ish.name, 26, 1, f);
@@ -787,16 +797,19 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	    copy_adjust(m->xxs[i].name, ish.name, 24);
 	}
 
+#define MAX(x) ((x) > 0xfffff ? 0xfffff : (x))
+
 	if (V(2) || (~ifh.flags & IT_USE_INST && V(1))) {
 	    if (strlen((char *) ish.name) || m->xxs[i].len > 1) {
 		report (
 		    "\n[%2X] %-26.26s %05x%c%05x %05x %05x %05x "
 		    "%02x%02x %02x%02x %5d ",
-		    i, ish.name,
+		    i, ifh.flags & IT_USE_INST ?
+				m->xxih[i].name : m->xxs[i].name,
 		    m->xxs[i].len,
 		    ish.flags & IT_SMP_16BIT ? '+' : ' ',
-		    m->xxs[i].lps, m->xxs[i].lpe,
-		    ish.sloopbeg, ish.sloopend,
+		    MAX(m->xxs[i].lps), MAX(m->xxs[i].lpe),
+		    MAX(ish.sloopbeg), MAX(ish.sloopend),
 		    ish.flags, ish.convert,
 		    ish.vol, ish.gvl, ish.c5spd
 		);
@@ -845,13 +858,14 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 		    if (o->big_endian)
 			xmp_cvt_sex(m->xxs[i].len, buf);
 		} else {
-		    itsex_decompress8(f, buf, m->xxs[i].len, ifh.cmwt == 0x0215);
+		    itsex_decompress8(f, buf, m->xxs[i].len, 
+					ish.convert & IT_CVT_DIFF);
 		}
 
 		xmp_drv_loadpatch(ctx, NULL, i, m->c4rate,
 				XMP_SMP_NOLOAD | cvt, &m->xxs[i], buf);
 		free (buf);
-		reportv(ctx, 0, "c");
+		reportv(ctx, 0, ish.convert & IT_CVT_DIFF ? "C" : "c");
 	    } else {
 		if (o->skipsmp) {
 		    fseek(f, m->xxs[i].len, SEEK_CUR);
