@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -34,7 +34,9 @@ static int liq_test(FILE *f, char *t, const int start)
 {
     char buf[15];
 
-    fread(buf, 1, 15, f);
+    if (fread(buf, 1, 15, f) < 15)
+	return -1;
+
     if (memcmp(buf, "Liquid Module:", 14))
 	return -1;
 
@@ -476,7 +478,7 @@ next_pattern:
 	m->xxi[i][0].pan = li.pan;
 	m->xxi[i][0].sid = i;
 
-	copy_adjust(m->xxih[i].name, li.name, 32);
+	copy_adjust(m->xxih[i].name, li.name, 31);
 
 	if ((V(1)) && (strlen((char *)m->xxih[i].name) || m->xxs[i].len)) {
 	    report ("\n[%2X] %-30.30s %05x%c%05x %05x %c %02x %02x %2d.%02d %5d ",
