@@ -25,6 +25,8 @@ void set_xxh_defaults(struct xxm_header *);
 void cvt_pt_event(struct xxm_event *, uint8 *);
 void disable_continue_fx(struct xxm_event *);
 void clean_s3m_seq(struct xxm_header *, uint8 *);
+int check_filename_case(char *, char *, char *, int);
+void get_instrument_path(struct xmp_context *, char *, char *, int);
 
 extern uint8 ord_xlat[];
 extern int arch_vol_table[];
