@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2011 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -8,6 +8,13 @@
 
 #include "common.h"
 
+
+#define MMD_INST_TYPES 9
+extern char *mmd_inst_type[];
+int mmd_get_8ch_tempo(int);
+void mmd_xlat_fx(struct xxm_event *, int, int, int);
+
+
 typedef int32 LONG;
 typedef uint32 ULONG;
 typedef int16 WORD;
