@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -27,7 +27,8 @@ static int mtm_test(FILE *f, char *t, const int start)
 {
     uint8 buf[4];
 
-    fread(buf, 1, 4, f);
+    if (fread(buf, 1, 4, f) < 4)
+	return -1;
     if (memcmp(buf, "MTM", 3))
 	return -1;
     if (buf[3] != 0x10)
