@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -257,7 +257,7 @@ static int ptm_load(struct xmp_context *ctx, FILE *f, const int start)
     }
     reportv(ctx, 0, "\n");
 
-    m->vol_xlat = ptm_vol;
+    m->vol_table = ptm_vol;
 
     for (i = 0; i < m->xxh->chn; i++)
 	m->xxc[i].pan = pfh.chset[i] << 4;
