@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -26,7 +26,9 @@ static int rad_test(FILE *f, char *t, const int start)
 {
 	char buf[16];
 
-	fread(buf, 1, 16, f);
+	if (fread(buf, 1, 16, f) < 16)
+		return -1;
+
 	if (memcmp(buf, "RAD by REALiTY!!", 16))
 		return -1;
 
