@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2011 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -343,7 +343,7 @@ void read_keyboard()
 
     /* Interactive commands */
 
-    if (nocmd)
+    if (nocmd || background)
 	return;
 
 #if defined HAVE_TERMIOS_H && !defined WIN32
@@ -402,11 +402,18 @@ void read_keyboard()
 	    break;
 	case ' ':	/* paused module */
 	    paused ^= 1;
+
 	    if (verbosity) {
-	    	fprintf (stderr, "%s",  paused ?
+	    	fprintf (stderr, "%s", paused ?
 				"] - PAUSED\b\b\b\b\b\b\b\b\b\b" :
 				"]         \b\b\b\b\b\b\b\b\b\b");
 	    }
+
+	    if (paused)
+		xmp_timer_stop(ctx);
+	    else
+		xmp_timer_restart(ctx);
+
 	    break;
 	case '1':
 	case '2':
@@ -526,7 +533,7 @@ int main(int argc, char **argv)
 
     if (opt->verbosity) {
 	fprintf(stderr, "Extended Module Player " VERSION "\n"
-	"Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr\n");
+	"Copyright (C) 1996-2011 Claudio Matsuoka and Hipolito Carraro Jr\n");
     }
 
     if (probeonly || (opt->verbosity)) {
