@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2011 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -103,7 +103,7 @@ static void list_wrap(char *s, int l, int r, int v)
 static void copyright_header()
 {
     printf("Extended Module Player " VERSION "\n"
-"Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr\n"
+"Copyright (C) 1996-2011 Claudio Matsuoka and Hipolito Carraro Jr\n"
 "Portions Copyright (C) 1996-1997 Takashi Iwai, (C) 1988 Tammo Hinrichs,\n"
 "(C) 1989 Rich Gopstein and Harris Corporation, (C) 1997 Bert Jahn, (C) 1998\n"
 "Sylvain Chipaux, (C) 1998,2000 Olivier Lapicque, (C) 1999 Tatsuyuki Satoh,\n"
@@ -195,6 +195,9 @@ static void usage(char *s, struct xmp_options *opt)
 "   -o --output-file name  Mix the module to file ('-' for stdout)\n"
 "   -u --unsigned          Set the mixer to use unsigned samples\n"
 
+"\nEnvironment options:\n"
+"   -I --instrument-path   Set pathname to external samples\n"
+
 "\nInformation options:\n"
 "   -h --help              Print a summary of the command line options\n"
 "   --load-only            Load module and exit\n"
@@ -213,7 +216,7 @@ static void usage(char *s, struct xmp_options *opt)
 void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx)
 {
     int optidx = 0;
-#define OPTIONS "8a:b:cD:d:Ff:hilM:mno:P:qRrS:s:T:t:uVvx:"
+#define OPTIONS "8a:b:cD:d:Ff:hI:ilM:mno:P:qRrS:s:T:t:uVvx:"
     static struct option lopt[] = {
 	{ "8bit",		 0, 0, '8' },
 	{ "amplify",		 1, 0, 'a' },
@@ -226,11 +229,12 @@ void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx
 	{ "click-filter",	 0, 0, 'F' },
 	{ "offset-bug-emulation",0, 0, OPT_FX9BUG },
 	{ "help",		 0, 0, 'h' },
+	{ "instrument-path",	 1, 0, 'I' },
 	{ "interpolate",	 0, 0, 'i' },
 	{ "load-only",	 	 0, 0, OPT_LOADONLY },
 	{ "loop",		 0, 0, 'l' },
-	{ "mute",		 1, 0, 'M' },
 	{ "mono",		 0, 0, 'm' },
+	{ "mute",		 1, 0, 'M' },
 	{ "nocmd",		 0, 0, OPT_NOCMD },
 	{ "nofilter",		 0, 0, OPT_NOFILTER },
 	{ "nearest",		 0, 0, 'n' },
@@ -303,6 +307,9 @@ void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx
 	case 'f':
 	    opt->freq = strtoul(optarg, NULL, 0);
 	    break;
+	case 'I':
+	    opt->ins_path = optarg;
+	    break;
 	case 'i':
 	    opt->flags |= XMP_CTL_ITPT;
 	    break;
