@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr.
+ * Copyright (C) 1996-2010 Claudio Matsuoka and Hipolito Carraro Jr.
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See docs/COPYING
@@ -157,11 +157,10 @@ void xmp_drv_close(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
 
-    xmp_drv_off(ctx);
     memset(d->cmute_array, 0, XMP_MAXCH * sizeof(int));
     d->driver->shutdown(ctx);
     free(d->patch_array);
-    /*synth_deinit();*/
+    synth_deinit();
 }
 
 
@@ -212,10 +211,11 @@ void xmp_drv_off(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
 
+    xmp_drv_writepatch(ctx, NULL);
+
     if (d->numchn < 1)
 	return;
 
-    xmp_drv_writepatch(ctx, NULL);
     d->curvoc = d->maxvoc = 0;
     d->numchn = 0;
     d->numtrk = 0;
@@ -225,7 +225,7 @@ void xmp_drv_off(struct xmp_context *ctx)
 }
 
 
-/*inline*/ void xmp_drv_clearmem(struct xmp_context *ctx)
+void xmp_drv_clearmem(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
 
@@ -577,7 +577,7 @@ int xmp_drv_cstat(struct xmp_context *ctx, int chn)
 }
 
 
-inline void xmp_drv_echoback(struct xmp_context *ctx, int msg)
+void xmp_drv_echoback(struct xmp_context *ctx, int msg)
 {
     struct xmp_driver_context *d = &ctx->d;
 
@@ -585,7 +585,7 @@ inline void xmp_drv_echoback(struct xmp_context *ctx, int msg)
 }
 
 
-inline int xmp_drv_getmsg(struct xmp_context *ctx)
+int xmp_drv_getmsg(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
 
@@ -593,7 +593,7 @@ inline int xmp_drv_getmsg(struct xmp_context *ctx)
 }
 
 
-inline void xmp_drv_bufdump(struct xmp_context *ctx)
+void xmp_drv_bufdump(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
     int i = xmp_smix_softmixer(ctx);
@@ -602,7 +602,7 @@ inline void xmp_drv_bufdump(struct xmp_context *ctx)
 }
 
 
-inline void xmp_drv_starttimer(struct xmp_context *ctx)
+void xmp_drv_starttimer(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
 
@@ -639,7 +639,7 @@ double xmp_drv_sync(struct xmp_context *ctx, double step)
 }
 
 
-inline void xmp_drv_bufwipe(struct xmp_context *ctx)
+void xmp_drv_bufwipe(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
 
@@ -659,7 +659,9 @@ int xmp_drv_writepatch(struct xmp_context *ctx, struct patch_info *patch)
 	d->driver->writepatch(ctx, patch);
 
 	for (num = XMP_MAXPAT; num--;) {
-	    free(d->patch_array[num]);
+	    if (d->patch_array[num]) {
+		free(d->patch_array[num]);
+	    }
 	    d->patch_array[num] = NULL;
 	}
 	return 0;
@@ -799,10 +801,11 @@ int xmp_drv_loadpatch(struct xmp_context *ctx, FILE *f, int id, int basefreq, in
 	memcpy(patch->data, buffer, xxs->len);
     } else {
 	int pos = ftell(f);
-	fread(s, 1, 5, f);
+	int num = fread(s, 1, 5, f);
+
 	fseek(f, pos, SEEK_SET);
 
-	if (!strncmp(s, "ADPCM", 5)) {
+	if (num == 5 && !memcmp(s, "ADPCM", 5)) {
 	    int x2 = xxs->len >> 1;
 	    char table[16];
 
@@ -883,7 +886,7 @@ int xmp_drv_loadpatch(struct xmp_context *ctx, FILE *f, int id, int basefreq, in
 }
 
 
-inline struct xmp_drv_info *xmp_drv_array()
+struct xmp_drv_info *xmp_drv_array()
 {
     return drv_array;
 }
