@@ -307,14 +307,15 @@ int _xmp_scan_module(struct xmp_context *ctx)
     row = break_row;
 
 end_module:
-    p->xmp_scan_num = o->start > ord ? 0: tab_cnt[ord][row];
-    p->xmp_scan_row = row;
-    p->xmp_scan_ord = ord;
+    p->scan_num = o->start > ord ? 0: tab_cnt[ord][row];
+    p->scan_row = row;
+    p->scan_ord = ord;
 
     free(loop_row);
     free(loop_stk);
 
-    for (ord = m->xxh->len; ord--; free (tab_cnt[ord]));
+    for (ord = m->xxh->len; ord--; )
+	free(tab_cnt[ord]);
     free(tab_cnt);
 
     clock -= clock_rst;
