@@ -315,7 +315,7 @@ static void mseek(InputPlayback *ipb, unsigned long time)
 			if (i > 0)
 				i--;
 			a = xmp_ord_set(ctx, i);
-#if __AUDACIOUS_PLUGIN_API__ < 13
+#if __AUDACIOUS_PLUGIN_API__ < 13 || __AUDACIOUS_PLUGIN_API__ >= 16
 			ipb->output->flush(p->m.xxo_info[i].time);
 #else
 			ipb->output->flush(p->m.xxo_info[i].time / 1000);
@@ -605,6 +605,7 @@ static void play_file(InputPlayback *ipb)
 		xmp_ip.set_info_text("Error loading mod");
 #endif
 		ipb->playing = 0;
+		xmp_close_audio(ctx);
 		return;
 	}
 
@@ -643,7 +644,11 @@ static gpointer play_loop(gpointer arg)
 	while (xmp_player_frame(ctx) == 0) {
 		xmp_get_buffer(ctx, &data, &size);
 
-#if __AUDACIOUS_PLUGIN_API__ >= 2
+#if __AUDACIOUS_PLUGIN_API__ >= 17
+		while (ipb->playing && ipb->output->buffer_playing())
+			g_usleep (10000);
+		ipb->output->write_audio(data, size);
+#elif __AUDACIOUS_PLUGIN_API__ >= 2
 		play_data.ipb->pass_audio(play_data.ipb, play_data.fmt,
 			play_data.nch, size, data, &play_data.ipb->playing);
 
@@ -939,15 +944,15 @@ static void config_ok(GtkWidget *widget, gpointer data)
 
 #define CFGWRITEINT(x) aud_cfg_db_set_int (cfg, "XMP", #x, xmp_cfg.x)
 
-	CFGWRITEINT (mixing_freq);
-	CFGWRITEINT (force8bit);
-	CFGWRITEINT (convert8bit);
-	CFGWRITEINT (modrange);
-	CFGWRITEINT (fixloops);
-	CFGWRITEINT (force_mono);
-	CFGWRITEINT (interpolation);
-	CFGWRITEINT (filter);
-	CFGWRITEINT (pan_amplitude);
+	CFGWRITEINT(mixing_freq);
+	CFGWRITEINT(force8bit);
+	CFGWRITEINT(convert8bit);
+	CFGWRITEINT(modrange);
+	CFGWRITEINT(fixloops);
+	CFGWRITEINT(force_mono);
+	CFGWRITEINT(interpolation);
+	CFGWRITEINT(filter);
+	CFGWRITEINT(pan_amplitude);
 
 	aud_cfg_db_close(cfg);
 
